/**
 * Lyceenne Maker J, created by Claire "Milena" Billaud
 * 25 avr. 09
 * Copyright (c) 2009, Claire "Milena" Billaud
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Claire Billaud, Milena, nor the
 *       names of other contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package models;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Defines a Lyceenne RPG character, with its characteristics, profiles, etc ...
 * @author milena
 * 
 */
public class Character {
	private String name;
	private int age;
	private Alignment alignment;
	private Map<String, Integer> characteristics;
	private List<Profile> profiles;
	private List<CharacterAbility> abilities;
	private List<CharacterSubject> subjects;
	private String history;
	private String reputationLevel;
	private String reputation;
	
	/**
	 * Constructor
	 * @param config the current config
	 */
	public Character(Config config)	{
		characteristics = new HashMap<String, Integer>();
		profiles = new ArrayList<Profile>();
		abilities = new ArrayList<CharacterAbility>();
		subjects = new ArrayList<CharacterSubject>();
		initCharacteristics();
		initAbilitiesFromConfig(config);
		recalcAllAbilities();
		//init first values of data
		age = 15;
		alignment = config.getAlignmentByName("Juste bonne");
		reputationLevel = "Inconnue/Transparente";
	}
	
	/**
	 * Initiates the character characteristics
	 * Characteristic list is standard and shouldn't change
	 *
	 */
	private void initCharacteristics()	{
		characteristics.put("PHY", 0);
		characteristics.put("DEX", 0);
		characteristics.put("INT", 0);
		characteristics.put("PER", 0);
		characteristics.put("CHA", 0);
		characteristics.put("SEN", 0);
	}
	
	/**
	 * Resets profiles list
	 * Must be called when a profile is selected/removed, before re-adding profiles from GUI
	 *
	 */
	public void resetAllProfiles()	{
		profiles.clear();
	}
	
	/**
	 * Initiates the character abilities from config abilities
	 * @param config the current config
	 */
	private void initAbilitiesFromConfig(Config config)	{
		for(Ability ability : config.getAbilities())	{
			CharacterAbility characterAbility = new CharacterAbility();
			characterAbility.setName(ability.getName());
			characterAbility.setCharacteristic(ability.getCharacteristic());
			characterAbility.setDefaultModifier(ability.getDefaultModifier());
			characterAbility.setUnusableByDefault(ability.isUnusableByDefault());
			characterAbility.setBonus(0);
			characterAbility.setMainAbility(false);
			abilities.add(characterAbility);
		}
	}
	
	/**
	 * Gets an ability from its name
	 * @param name the exact name of the ability
	 * @return the ability if it exists, or null
	 */
	public CharacterAbility getAbilityByName(String name)	{
		for(CharacterAbility ability : abilities)	{
			if(ability.getName().equals(name))	{
				return ability;
			}
		}
		return null;
	}
	
	/**
	 * Recalculates all ability scores.
	 * Must be called :
	 * - when a characteristic value is changed
	 * - when a profile is changed
	 * - when a bonus is added to an ability
	 * - when an ability is set as main ability
	 *
	 */
	public void recalcAllAbilities()	{
		for(CharacterAbility characterAbility : abilities)	{
			int totalScore = 0;
			if(!characterAbility.isUnusableByDefault() || (characterAbility.isUnusableByDefault() && characterAbility.isMainAbility()))	{
				totalScore = characteristics.get(characterAbility.getCharacteristic()) + characterAbility.getBonus();
				if(!characterAbility.isMainAbility())	{
					totalScore = totalScore + characterAbility.getDefaultModifier();
				}
				for(Profile profile : profiles)	{
					if(profile.getAbilityModifiers().containsKey(characterAbility.getName()))	{
						totalScore = totalScore + profile.getAbilityModifiers().get(characterAbility.getName());
						break;
					}
				}
				//adjusts score if not between 1 and 10
				if(totalScore > 10)	totalScore = 10;
				if(totalScore <= 0)	totalScore = 1;
			}
			characterAbility.setTotalScore(totalScore);
		}
	}
	
	/**
	 * Resets all ability bonuses
	 * Must be called when age is changed (changes generation points)
	 *
	 */
	public void resetAllAbilityBonuses()	{
		for(CharacterAbility characterAbility : abilities)	{
			characterAbility.setBonus(0);
		}
		recalcAllAbilities();
	}
	
	/**
	 * Resets subjects list
	 * Must be called when a subject is changed, before re-adding subjects from GUI, or when subjects number is changed
	 *
	 */
	public void resetAllSubjects()	{
		subjects.clear();
	}
	
	/**
	 * Adds a subject from its name
	 * @param config the current config
	 * @param subjectName the name of the subject to add
	 */
	public CharacterSubject addSubjectFromConfig(Config config, String subjectName)	{
		Subject subject = config.getSubjectByName(subjectName);
		CharacterSubject characterSubject = new CharacterSubject();
		characterSubject.setName(subject.getName());
		characterSubject.setAssociatedAbility(subject.getAssociatedAbility());
		characterSubject.setBonus(0);
		subjects.add(characterSubject);
		return characterSubject;
	}
	
	/**
	 * Gets a subject from its name
	 * @param name the exact name of the subject
	 * @return the subject if it exists, or null
	 */
	public CharacterSubject getSubjectByName(String name)	{
		for(CharacterSubject subject : subjects)	{
			if(subject.getName().equals(name))	{
				return subject;
			}
		}
		return null;
	}
	
	/**
	 * Recalculates all subjects scores
	 * Normally invoked when all characteristics and abilities are definitively set.
	 * Must be called :
	 * - when a subject value is changed
	 *
	 */
	public void recalcAllSubjects()	{
		for(CharacterSubject characterSubject : subjects)	{
			int totalScore = 0;
			if(characterSubject.getAssociatedAbility() != null)	{
				for(CharacterAbility ability : abilities)	{
					if(characterSubject.getAssociatedAbility().equals(ability.getName()))	{
						totalScore = ability.getTotalScore() * 2;
						break;
					}
				}
			}else	{
				totalScore = characteristics.get("INT") + characterSubject.getBonus();
			}
			if(totalScore > 20)	totalScore = 20;
			characterSubject.setTotalScore(totalScore);
		}
	}
	
	public List<CharacterAbility> getAbilities() {
		return abilities;
	}
	public void setAbilities(List<CharacterAbility> abilities) {
		this.abilities = abilities;
	}
	public int getAge() {
		return age;
	}
	public void setAge(int age) {
		this.age = age;
	}
	public Alignment getAlignment() {
		return alignment;
	}
	public void setAlignment(Alignment alignment) {
		this.alignment = alignment;
	}
	public Map<String, Integer> getCharacteristics() {
		return characteristics;
	}
	public void setCharacteristics(Map<String, Integer> characteristics) {
		this.characteristics = characteristics;
	}
	public String getHistory() {
		return history;
	}
	public void setHistory(String history) {
		this.history = history;
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public List<Profile> getProfiles() {
		return profiles;
	}
	public void setProfiles(List<Profile> profiles) {
		this.profiles = profiles;
	}
	public List<CharacterSubject> getSubjects() {
		return subjects;
	}
	public void setSubjects(List<CharacterSubject> subjects) {
		this.subjects = subjects;
	}

	public String getReputation() {
		return reputation;
	}

	public void setReputation(String reputation) {
		this.reputation = reputation;
	}

	public String getReputationLevel() {
		return reputationLevel;
	}

	public void setReputationLevel(String reputationLevel) {
		this.reputationLevel = reputationLevel;
	}
	
}
