/**
 * Lyceenne Maker J, created by Claire "Milena" Billaud
 * 25 avr. 09
 * Copyright (c) 2009, Claire "Milena" Billaud
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Claire Billaud, Milena, nor the
 *       names of other contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package models;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import utilities.FileUtils;

/**
 * Configuration of the program. Loads all existing profiles, abilities, subjects ... from files.
 * @author milena
 * 
 */
public class Config {
	private List<Alignment> alignments;
	private List<Profile> profiles;
	private List<Ability> abilities;
	private List<Subject> subjects;
	private List<String> reputationLevels;
	
	/**
	 * Constructor
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public Config() throws FileNotFoundException, IOException	{
		alignments = new ArrayList<Alignment>();
		profiles = new ArrayList<Profile>();
		abilities = new ArrayList<Ability>();
		subjects = new ArrayList<Subject>();
		reputationLevels = new ArrayList<String>();
		loadConfigFromFiles();
		loadAlignments();
		loadReputationLevels();
	}
	
	/**
	 * Loads all attributes from lib files
	 * Called by constructor only
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	private void loadConfigFromFiles() throws FileNotFoundException, IOException	{
		List<String> profilesList = FileUtils.readLinesFromFile("profiles.txt");
		for(String profileDescriptor : profilesList){
			loadProfile(profileDescriptor);
		}
		List<String> abilitiesList = FileUtils.readLinesFromFile("abilities.txt");
		for(String abilityDescriptor : abilitiesList)	{
			loadAbility(abilityDescriptor);
		}
		List<String> subjectsList = FileUtils.readLinesFromFile("subjects.txt");
		for(String subjectDescriptor : subjectsList)	{
			loadSubject(subjectDescriptor);
		}
	}
	
	/**
	 * Loads a profile from a line of profiles.txt file
	 * @param profileDescriptor a line of the file describing a profile
	 */
	private void loadProfile(String profileDescriptor)	{
		Profile profile = new Profile();
		String[] profileElements = profileDescriptor.split(",");
		profile.setName(profileElements[0]);
		profile.setType(profileElements[1]);
		if(profileElements.length > 2)	{
			//first ability modifier
			profile.getAbilityModifiers().put(profileElements[2], Integer.parseInt(profileElements[3]));
		}
		profiles.add(profile);
	}
	
	/**
	 * Loads an ability from a line of abilities.txt file
	 * @param abilityDescriptor a line of the file describing an ability
	 */
	private void loadAbility(String abilityDescriptor)	{
		Ability ability = new Ability();
		String[] abilityElements = abilityDescriptor.split(",");
		ability.setName(abilityElements[0]);
		ability.setCharacteristic(abilityElements[1]);
		if("X".equals(abilityElements[2]))	{
			ability.setDefaultModifier(0);
			ability.setUnusableByDefault(true);
		}else	{
			ability.setDefaultModifier(Integer.parseInt(abilityElements[2]));
			ability.setUnusableByDefault(false);
		}
		abilities.add(ability);
	}
	
	/**
	 * Loads a subject from a line of the subjects.txt file
	 * @param subjectDescriptor a line of the file describing a subject
	 */
	private void loadSubject(String subjectDescriptor)	{
		Subject subject = new Subject();
		String[] subjectElements = subjectDescriptor.split(",");
		subject.setName(subjectElements[0]);
		if(subjectElements.length > 1)	{
			subject.setAssociatedAbility(subjectElements[1]);
		}
		subjects.add(subject);
	}
	
	/**
	 * Loads alignments
	 * Alignments list is standard and shouldn't change
	 *
	 */
	private void loadAlignments()	{
		alignments.add(new Alignment("Juste bonne", 2, 4, 0, 2));
		alignments.add(new Alignment("Impartiale bonne", 2, 4, 0, 2));
		alignments.add(new Alignment("Rebelle bonne", 2, 4, 0, 2));
		alignments.add(new Alignment("Juste neutre", 1, 3, 1, 3));
		alignments.add(new Alignment("Impartiale neutre", 1, 3, 1, 3));
		alignments.add(new Alignment("Rebelle neutre", 1, 3, 1, 3));
		alignments.add(new Alignment("Juste mauvaise", 0, 2, 2, 4));
		alignments.add(new Alignment("Impartiale mauvaise", 0, 2, 2, 4));
		alignments.add(new Alignment("Rebelle mauvaise", 0, 2, 2, 4));
	}
	
	/**
	 * Gets an alignment from its name
	 * @param name the exact name of the alignment
	 * @return the alignment if it exists, or null
	 */
	public Alignment getAlignmentByName(String name)	{
		for(Alignment alignment : alignments)	{
			if(alignment.getName().equals(name))	{
				return alignment;
			}
		}
		return null;
	}
	
	/**
	 * Gets a profile from its name
	 * @param name the exact name of the profile
	 * @return the profile if it exists, or null
	 */
	public Profile getProfileByName(String name){
		for(Profile profile : profiles)	{
			if(profile.getName().equals(name))	{
				return profile;
			}
		}
		return null;
	}
	
	/**
	 * Gets an ability from its name
	 * @param name the exact name of the ability
	 * @return the ability if it exists, or null
	 */
	public Ability getAbilityByName(String name)	{
		for(Ability ability : abilities)	{
			if(ability.getName().equals(name))	{
				return ability;
			}
		}
		return null;
	}
	
	/**
	 * Gets a subject from its name
	 * @param name the exact name of the subject
	 * @return the subject if it exists, or null
	 */
	public Subject getSubjectByName(String name)	{
		for(Subject subject : subjects)	{
			if(subject.getName().equals(name))	{
				return subject;
			}
		}
		return null;
	}
	
	/**
	 * Loads reputation levels
	 *
	 */
	public void loadReputationLevels()	{
		reputationLevels.add("Inconnue/Transparente");
		reputationLevels.add("Discrète");
		reputationLevels.add("Connue");
		reputationLevels.add("Incontournable");
	}

	public List<Alignment> getAlignments()	{
		return alignments;
	}
	
	public void setAlignments(List<Alignment> alignments)	{
		this.alignments = alignments;
	}
	
	public List<Ability> getAbilities() {
		return abilities;
	}

	public void setAbilities(List<Ability> abilities) {
		this.abilities = abilities;
	}

	public List<Profile> getProfiles() {
		return profiles;
	}

	public void setProfiles(List<Profile> profiles) {
		this.profiles = profiles;
	}

	public List<Subject> getSubjects() {
		return subjects;
	}

	public void setSubjects(List<Subject> subjects) {
		this.subjects = subjects;
	}

	public List<String> getReputationLevels() {
		return reputationLevels;
	}

	public void setReputationLevels(List<String> reputationLevels) {
		this.reputationLevels = reputationLevels;
	}
	
}
