/**
 * Lyceenne Maker J, created by Claire "Milena" Billaud
 * 1 mai 09
 * Copyright (c) 2009, Claire "Milena" Billaud
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Claire Billaud, Milena, nor the
 *       names of other contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package utilities;

import java.util.HashSet;
import java.util.Set;

import models.CharacterSubject;
import models.Profile;

/**
 * Class containing miscellaneous checks for character creation
 * @author milena
 *
 */
public class CharacterRules {
	
	/**
	 * Checks if the number of profiles of a character matches its alignment (checks only max profiles)
	 * @param character the character to check, with its profiles and alignment
	 * @return true if it is OK, false else
	 */
	public static boolean checkAlignmentAndMaxProfiles(models.Character character)	{
		int totalPositive = 0;
		int totalNegative = 0;
		for(Profile profile : character.getProfiles())	{
			if("Positif".equals(profile.getType()))	{
				totalPositive += 1;
			}else if("Négatif".equals(profile.getType()))	{
				totalNegative += 1;
			}
		}
		if(character.getAlignment().getMaxPositiveProfiles() < totalPositive)	{
			return false;
		}
		if(character.getAlignment().getMaxNegativeProfiles() < totalNegative)	{
			return false;
		}
		return true;
	}
	
	/**
	 * Checks if the number of profiles of a character matches its alignment
	 * Complete check, to be called when Next button is clicked.
	 * @param character the character to check, with its profiles and alignment
	 * @return true if it is OK, false else
	 */
	public static boolean checkAlignmentAndProfiles(models.Character character)	{
		int totalPositive = 0;
		int totalNegative = 0;
		for(Profile profile : character.getProfiles())	{
			if("Positif".equals(profile.getType()))	{
				totalPositive += 1;
			}else if("Négatif".equals(profile.getType()))	{
				totalNegative += 1;
			}
		}
		if(character.getAlignment().getMaxPositiveProfiles() < totalPositive || character.getAlignment().getMinPositiveProfiles() > totalPositive)	{
			return false;
		}
		if(character.getAlignment().getMaxNegativeProfiles() < totalNegative || character.getAlignment().getMinNegativeProfiles() > totalNegative)	{
			return false;
		}
		return true;
	}
	
	/**
	 * Checks if there is no duplicate profile in a character
	 * @param character the character to check
	 * @return true if it is OK, false else
	 */
	public static boolean hasNoDoubleProfiles(models.Character character)	{
		Set<Profile> uniqueProfileSet = new HashSet<Profile>();
		boolean isOk = true;
		for(Profile profile : character.getProfiles())	{
			if(!uniqueProfileSet.add(profile))	{
				isOk = false;
				break;
			}
		}
		return isOk;
	}
	
	/**
	 * Checks if there is no duplicate subject in a character
	 * Note : it is different from hasNoDoubleProfiles, as CharacterSubjects are created and not linked from Config.
	 * @param character the character to check
	 * @return true if it is OK, false else
	 */
	public static boolean hasNoDoubleSubjects(models.Character character){
		Set<String> uniqueSubjectSet = new HashSet<String>();
		boolean isOk = true;
		for(CharacterSubject subject : character.getSubjects())	{
			if(!uniqueSubjectSet.add(subject.getName()))	{
				isOk = false;
				break;
			}
		}
		return isOk;
	}
}
